@@ -151,7 +151,8 @@ calculateAverageSpectra <- function(x, evNum=1, calibration=NULL, wl=512,
     specData <- binToSpecMat(binData, channel=channel, wl=wl, sr=sr, freq=freq,
                              calFun=calFun, mode=mode, noise=FALSE, decimate=decimate,
                              filterfrom_khz=filterfrom_khz, filterto_khz = filterto_khz, ...)
-    specData <- specData[!sapply(specData, is.null)]
+    zeroClip <- sapply(specData, function(x) all(is.na(x)))
+    specData <- specData[!zeroClip]
     specMat <- matrix(NA,nrow=length(specData[[1]]), ncol=length(specData))
 
     for(i in seq_along(specData)) {
@@ -162,7 +163,7 @@ calculateAverageSpectra <- function(x, evNum=1, calibration=NULL, wl=512,
     noiseData <- binToSpecMat(binData, channel=channel, wl=wl, sr=sr, freq=freq,
                               calFun=calFun, mode=mode, noise=TRUE, decimate=decimate,
                               filterfrom_khz=filterfrom_khz, filterto_khz = filterto_khz, ...)
-    noiseData <- noiseData[!sapply(noiseData, is.null)]
+    noiseData <- noiseData[!zeroClip]
     noiseMat <- matrix(NA,nrow=length(noiseData[[1]]), ncol=length(noiseData))
 
     for(i in seq_along(noiseData)) {
