@@ -27,7 +27,7 @@
 #'   as long as it has a time associated with it. Data will be attached to
 #'   detector dataframes for \code{mode='detection'} or to the event "measures"
 #'   location for \code{mode='event'} (this is where \link{calculateICI} and
-#'   \link{matchEnvData} store their event data). These can be accessed with the
+#'   \link{matchEnvData,AcousticStudy-method} store their event data). These can be accessed with the
 #'   \link{getMeasures} function and are also exported in the various "getXXX"
 #'   functions (\link{getClickData} etc.) if \code{measures=TRUE} (default).
 #'
@@ -124,9 +124,9 @@ matchTimeData <- function(x, data, mode=c('event', 'detection'), thresh=Inf, int
                 newNa <- sapply(colnames(thisDet)[!colnames(thisDet) %in% oldCols], function(c) {
                     anyNA(thisDet[[c]])
                 })
-                newNa <- map(colnames(thisDet)[!colnames(thisDet) %in% oldCols],
-                             \(c) is.na(thisDet[[c]])
-                ) %>%
+                newNa <- map(colnames(thisDet)[!colnames(thisDet) %in% oldCols], function(c) {
+                             is.na(thisDet[[c]])
+                }) %>%
                     reduce(`|`)
                 if(any(newNa)) {
                     diffs <- c(diffs, thisDet$timeDiff[newNa])
